package jp.littlesoft.gwttest1.client;

import jp.littlesoft.gwttest1.client.ImageSearchService;
import jp.littlesoft.gwttest1.client.ImageSearchServiceAsync;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.Frame;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.SourcesTableEvents;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.TableListener;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.WindowResizeListener;


/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class ImageSearchClient implements EntryPoint , WindowResizeListener, 
                                          ClickListener, TableListener {
  
  DockPanel dockPanel = new DockPanel();
  HorizontalPanel header = new HorizontalPanel();
  HorizontalPanel footer = new HorizontalPanel();
  
  TextBox keyword = new TextBox();
  Button searchButton = new Button("検索");
  Label label1 = new Label("画像を検索するキーワード ：");
  Label label2 = new Label("");
  Label label3 = new Label("LittleSoft Corporation");

  MyGrid resultGrid = new MyGrid(0,4);
  MyScrollPanel resultGridScroll = new MyScrollPanel(resultGrid, this);
  Frame frame = new Frame();

  ImageSearchServiceAsync searchService;
  int resultSetSize = 16;
  int resultSetPosition = 1;
  int lastSelected = -1;
  boolean loading = false;


  /**
   * This is the entry point method.
   */
  public void onModuleLoad() {
    
    searchService = (ImageSearchServiceAsync) GWT.create(ImageSearchService.class);
    ServiceDefTarget target = (ServiceDefTarget) searchService;
    target.setServiceEntryPoint("/test/search");
    
    RootPanel.get().add(dockPanel);

    keyword.setWidth("240px");
    
    header.add(label1);
    header.add(keyword);
    header.add(searchButton);
    header.add(label2);
    
    header.setCellVerticalAlignment(label1, HorizontalPanel.ALIGN_MIDDLE);
    header.setCellVerticalAlignment(keyword, HorizontalPanel.ALIGN_MIDDLE);
    header.setCellVerticalAlignment(searchButton, HorizontalPanel.ALIGN_MIDDLE);
    header.setCellVerticalAlignment(label2, HorizontalPanel.ALIGN_MIDDLE);

    dockPanel.add(header, DockPanel.NORTH );
    dockPanel.setCellHeight(header,"32px");
    dockPanel.setCellHorizontalAlignment(header, DockPanel.ALIGN_LEFT);
    dockPanel.setCellVerticalAlignment(header, DockPanel.ALIGN_MIDDLE);
    
    footer.add(label3);
    dockPanel.add(footer, DockPanel.SOUTH );
    dockPanel.setCellHeight(footer,"32px");
    dockPanel.setCellHorizontalAlignment(footer, DockPanel.ALIGN_CENTER);
    dockPanel.setCellVerticalAlignment(footer, DockPanel.ALIGN_MIDDLE);

    dockPanel.add(resultGridScroll, DockPanel.WEST );
    dockPanel.add(frame, DockPanel.CENTER );
    dockPanel.setSize("100%","100%");
    
    resultGrid.setCellSpacing(0);
    resultGrid.setCellPadding(0);
    resultGrid.setVisible(false);
    
    //スタイル名の設定
    dockPanel.addStyleName("dock");
    resultGrid.addStyleName("grid");
    resultGridScroll.addStyleName("scroll");
    frame.addStyleName("frame");

    Window.addWindowResizeListener(this);
    onWindowResized(Window.getClientWidth(), Window.getClientHeight());
    
    searchButton.addClickListener(this);
    
    resultGrid.addTableListener(this);

  }

  public void onWindowResized(int width, int height) {
    resultGridScroll.setPixelSize((int)(width*0.4)-2, (height-66));
    frame.setPixelSize((int)(width*0.6)-2, (height-66));
  }
  
  public void onClick(Widget sender) {
    doSearch(false);
  }  
  
  void doSearch(final boolean fetchNext) {
    if (this.loading) return;
    if (keyword.getText() == null || keyword.getText().length()==0) {
      Window.alert("キーワードが必要です。");
      keyword.setFocus(true);
      return;
    }
    this.loading = true;
    if (fetchNext) {
      this.resultSetPosition = this.resultSetPosition + this.resultSetSize;
    } else {
      this.resultSetPosition = 1;
      resultGrid.resizeRows(0);
      resultGridScroll.setScrollPosition(0);
    }
    searchService.execute(this.resultSetPosition, this.resultSetSize, 
        keyword.getText(), new ImageSearchCallback(this) );
  }
  
  class ImageSearchCallback implements AsyncCallback {

    ImageSearchClient client;

    public ImageSearchCallback(ImageSearchClient imageSearch) {
      this.client = imageSearch;
    }
    
    public void onFailure(Throwable caught) {
      Window.alert("onFailure:" + caught.toString());
      this.client.resultGrid.setVisible(false);
      this.client.loading = false;
    }

    public void onSuccess(Object response) {
      try {
        ImageSearchResultSet resultSet = (ImageSearchResultSet) response;
        this.client.label2.setText(
          keyword.getText() + "で検索した結果 " + resultSet.firstResultPosition + " - " +
          resultSet.totalResultsReturned + "件 / 約" + resultSet.totalResultsAvailable + "件");
        
        Grid grid = this.client.resultGrid;
        for (int i=0; i<resultSet.array.size(); i++) {
          ImageSearchResultSet.Result rs = (ImageSearchResultSet.Result)resultSet.array.get(i);
          int row = grid.getRowCount();
          grid.resizeRows(row + 1);
          grid.setWidget(row, 0, new Label("No." + (row + 1)));
          grid.setWidget(row,1, new Image(rs.thumbnail.url));
          grid.setWidget(row,2, new Label(rs.summary));
          grid.setWidget(row,3, new Label(rs.refererUrl));
          grid.getWidget(row,3).setVisible(false);
        }
        grid.setVisible(grid.getRowCount()>0);
      } catch (Exception x) {
        x.printStackTrace();
        Window.alert(x.toString());
      } finally {
        this.client.loading = false;
      }
    }
  }
  
  class MyGrid extends Grid {
    
    public MyGrid(int rows, int columns) {
      super(rows, columns);
    }
    
    public void resizeRows(int rows) { 
      if (numRows == rows) return; 
      if (rows < 0) { 
        throw new IndexOutOfBoundsException("Cannot set number of rows to" + rows); 
      } 
      while (numRows < rows) { 
        // More rows. Add them where necessary. 
        insertRow(numRows); 
        insertCells(numRows, 0, numColumns); 
        ++numRows; 
      } 
      while (numRows > rows) { 
        // Fewer rows. Remove extraneous ones. 
        removeRow(--numRows); 
      } 
    } 
  }  
  
  class MyScrollPanel extends ScrollPanel {

    Widget grid;
    ImageSearchClient client;
    
    public MyScrollPanel(Widget child, ImageSearchClient client) {
      super(child);
      this.client = client;
      this.grid = child;
      this.sinkEvents(Event.ONSCROLL);
    }
    
    public void onBrowserEvent(Event event) { 
      if (DOM.eventGetType(event) == Event.ONSCROLL) { 
        if ( (this.grid.getOffsetHeight() - this.getOffsetHeight()) 
                < (this.getOffsetHeight() + this.getScrollPosition()) ) {
          client.doSearch(true);
        }
      } else { 
        super.onBrowserEvent(event); 
      } 
    } 
  }
  
  public void onCellClicked(SourcesTableEvents sender, int row, int cell){
    frame.setUrl(((Label)resultGrid.getWidget(row, 3)).getText());
    try {
       resultGrid.getRowFormatter().removeStyleName(this.lastSelected, "selected"); 
    } catch (Exception x) {}
    resultGrid.getRowFormatter().addStyleName(row, "selected");
    this.lastSelected = row;
  }

}
